@@ -81,10 +81,13 @@ struct rte;
 
 int f_run(struct filter *filter, struct rte **rte, struct ea_list **tmp_attrs, struct linpool *tmp_pool, int flags);
 int f_eval_int(struct f_inst *expr);
+u32 f_eval_asn(struct f_inst *expr);
+
 char *filter_name(struct filter *filter);
 int filter_same(struct filter *new, struct filter *old);
 
 int i_same(struct f_inst *f1, struct f_inst *f2);
+void f_prefix_get_bounds(struct f_prefix *px, int *l, int *h);
 
 int val_compare(struct f_val v1, struct f_val v2);
 void val_print(struct f_val v);
