@@ -20,6 +20,7 @@ typedef struct birdsock {
   unsigned sport, dport;		/* 0 = unspecified (for IP: protocol type) */
   int tos;				/* TOS and priority, -1 = default */
   int ttl;				/* Time To Live, -1 = default */
+  u32 flags;
   struct iface *iface;			/* Interface; specify this for broad/multicast sockets */
 
   byte *rbuf, *rpos;			/* NULL=allocate automatically */
@@ -50,6 +51,7 @@ void sk_reallocate(sock *);		/* Free and allocate tbuf & rbuf */
 void sk_dump_all(void);
 int sk_set_ttl(sock *s, int ttl);	/* Set TTL for given socket */
 int sk_set_md5_auth(sock *s, ip_addr a, char *passwd);	/* Add or remove security associations for given passive socket */
+int sk_rx_ready(sock *s);
 
 static inline int
 sk_send_buffer_empty(sock *sk)
@@ -57,6 +59,12 @@ sk_send_buffer_empty(sock *sk)
 	return sk->tbuf == sk->tpos;
 }
 
+
+/* Socket flags */
+
+#define SKF_V6ONLY	1	/* Use  IPV6_V6ONLY socket option */
+
+
 /*
  *	Socket types		     SA SP DA DP IF  TTL SendTo	(?=may, -=must not, *=must)
  */
