@@ -234,7 +234,7 @@ bgp_create_update(struct bgp_conn *conn, byte *buf)
 {
   struct bgp_proto *p = conn->bgp;
   struct bgp_bucket *buck;
-  int size;
+  int size, second;
   int remains = BGP_MAX_PACKET_LENGTH - BGP_HEADER_LENGTH - 4;
   byte *w, *tmp, *tstart;
   ip_addr *ipp, ip, ip_ll;
@@ -292,7 +292,9 @@ bgp_create_update(struct bgp_conn *conn, byte *buf)
 	  nh = ea_find(buck->eattrs, EA_CODE(EAP_BGP, BA_NEXT_HOP));
 	  ASSERT(nh);
 
-	  /* We have two addresses here in 'nh'. Really. */
+	  /* We have two addresses here in 'nh'. Really.
+	     Unless NEXT_HOP was modified by filter */
+	  second = (nh->u.ptr->length == NEXT_HOP_LENGTH);
 	  ipp = (ip_addr *) nh->u.ptr->data;
 	  ip = ipp[0];
 	  ip_ll = IPA_NONE;
@@ -316,7 +318,7 @@ bgp_create_update(struct bgp_conn *conn, byte *buf)
 	      n = neigh_find(&p->p, &ip, 0);
 	      if (n && n->iface == p->neigh->iface)
 		{
-		  if (ipa_nonzero(ipp[1]))
+		  if (second && ipa_nonzero(ipp[1]))
 		    ip_ll = ipp[1];
 		  else
 		    {
@@ -488,7 +490,6 @@ bgp_parse_capabilities(struct bgp_conn *conn, byte *opt, int len)
 {
   struct bgp_proto *p = conn->bgp;
   int cl;
-  u32 as;
 
   while (len > 0)
     {
@@ -570,7 +571,6 @@ bgp_rx_open(struct bgp_conn *conn, byte *pkt, int len)
 {
   struct bgp_conn *other;
   struct bgp_proto *p = conn->bgp;
-  struct bgp_config *cf = p->cf;
   unsigned hold;
   u16 base_as;
   u32 id;
@@ -602,7 +602,17 @@ bgp_rx_open(struct bgp_conn *conn, byte *pkt, int len)
     log(L_WARN "%s: Peer advertised inconsistent AS numbers", p->p.name);
 
   if (conn->advertised_as != p->remote_as)
-    { bgp_error(conn, 2, 2, (byte *) &(conn->advertised_as), -4); return; }
+    {
+      if (conn->peer_as4_support)
+	{
+	  u32 val = htonl(conn->advertised_as);
+	  bgp_error(conn, 2, 2, (byte *) &val, 4);
+	}
+      else
+	bgp_error(conn, 2, 2, pkt+20, 2);
+
+      return;
+    }
 
   /* Check the other connection */
   other = (conn == &p->outgoing_conn) ? &p->incoming_conn : &p->outgoing_conn;
@@ -619,14 +629,14 @@ bgp_rx_open(struct bgp_conn *conn, byte *pkt, int len)
 	{
 	  /* Should close the other connection */
 	  BGP_TRACE(D_EVENTS, "Connection collision, giving up the other connection");
-	  bgp_error(other, 6, 0, NULL, 0);
+	  bgp_error(other, 6, 7, NULL, 0);
 	  break;
 	}
       /* Fall thru */
     case BS_ESTABLISHED:
       /* Should close this connection */
       BGP_TRACE(D_EVENTS, "Connection collision, giving up this connection");
-      bgp_error(conn, 6, 0, NULL, 0);
+      bgp_error(conn, 6, 7, NULL, 0);
       return;
     default:
       bug("bgp_rx_open: Unknown state");
@@ -705,7 +715,7 @@ bgp_do_rx_update(struct bgp_conn *conn,
       DECODE_PREFIX(withdrawn, withdrawn_len);
       DBG("Withdraw %I/%d\n", prefix, pxlen);
       if (n = net_find(p->p.table, prefix, pxlen))
-	rte_update(p->p.table, n, &p->p, NULL);
+	rte_update(p->p.table, n, &p->p, &p->p, NULL);
     }
 
   if (!attr_len && !nlri_len)		/* shortcut */
@@ -724,14 +734,20 @@ bgp_do_rx_update(struct bgp_conn *conn,
 	  n = net_get(p->p.table, prefix, pxlen);
 	  e->net = n;
 	  e->pflags = 0;
-	  rte_update(p->p.table, n, &p->p, e);
+	  rte_update(p->p.table, n, &p->p, &p->p, e);
+	  if (bgp_apply_limits(p) < 0)
+	    goto bad2;
 	}
+      rta_free(a);
     }
-bad:
+
+  return;
+
+ bad:
+  bgp_error(conn, 3, err, NULL, 0);
+ bad2:
   if (a)
     rta_free(a);
-  if (err)
-    bgp_error(conn, 3, err, NULL, 0);
   return;
 }
 
@@ -783,7 +799,7 @@ bgp_do_rx_update(struct bgp_conn *conn,
 	  DECODE_PREFIX(x, len);
 	  DBG("Withdraw %I/%d\n", prefix, pxlen);
 	  if (n = net_find(p->p.table, prefix, pxlen))
-	    rte_update(p->p.table, n, &p->p, NULL);
+	    rte_update(p->p.table, n, &p->p, &p->p, NULL);
 	}
     }
 
@@ -824,7 +840,9 @@ bgp_do_rx_update(struct bgp_conn *conn,
 	      n = net_get(p->p.table, prefix, pxlen);
 	      e->net = n;
 	      e->pflags = 0;
-	      rte_update(p->p.table, n, &p->p, e);
+	      rte_update(p->p.table, n, &p->p, &p->p, e);
+	      if (bgp_apply_limits(p) < 0)
+		goto bad2;
 	    }
 	  rta_free(a);
 	}
@@ -832,8 +850,9 @@ bgp_do_rx_update(struct bgp_conn *conn,
 
   return;
 
-bad:
+ bad:
   bgp_error(conn, 3, 9, start, len0);
+ bad2:
   if (a)
     rta_free(a);
   return;
@@ -948,14 +967,15 @@ bgp_error_dsc(byte *buff, unsigned code, unsigned subcode)
 }
 
 void
-bgp_log_error(struct bgp_proto *p, char *msg, unsigned code, unsigned subcode, byte *data, unsigned len)
+bgp_log_error(struct bgp_proto *p, u8 class, char *msg, unsigned code, unsigned subcode, byte *data, unsigned len)
 {
   const byte *name;
   byte namebuf[32];
   byte *t, argbuf[36];
   unsigned i;
 
-  if (code == 6 && !subcode)		/* Don't report Cease messages */
+  /* Don't report Cease messages generated by myself */
+  if (code == 6 && class == BE_BGP_TX)
     return;
 
   name = bgp_error_dsc(namebuf, code, subcode);
@@ -964,11 +984,19 @@ bgp_log_error(struct bgp_proto *p, char *msg, unsigned code, unsigned subcode, b
     {
       *t++ = ':';
       *t++ = ' ';
+
+      if ((code == 2) && (subcode == 2) && ((len == 2) || (len == 4)))
+	{
+	  /* Bad peer AS - we would like to print the AS */
+	  t += bsprintf(t, "%d", (len == 2) ? get_u16(data) : get_u32(data));
+	  goto done;
+	}
       if (len > 16)
 	len = 16;
       for (i=0; i<len; i++)
 	t += bsprintf(t, "%02x", data[i]);
     }
+ done:
   *t = 0;
   log(L_REMOTE "%s: %s: %s%s", p->p.name, msg, name, argbuf);
 }
@@ -985,10 +1013,10 @@ bgp_rx_notification(struct bgp_conn *conn, byte *pkt, int len)
 
   unsigned code = pkt[19];
   unsigned subcode = pkt[20];
-  int delay = 1;
+  int err = (code != 6);
 
-  bgp_log_error(conn->bgp, "Received error notification", code, subcode, pkt+21, len-21);
-  bgp_store_error(conn->bgp, conn, BE_BGP_RX, (code << 16) | subcode);
+  bgp_log_error(p, BE_BGP_RX, "Received", code, subcode, pkt+21, len-21);
+  bgp_store_error(p, conn, BE_BGP_RX, (code << 16) | subcode);
 
 #ifndef IPV6
   if ((code == 2) && ((subcode == 4) || (subcode == 7))
@@ -1005,14 +1033,19 @@ bgp_rx_notification(struct bgp_conn *conn, byte *pkt, int len)
     {
       /* We try connect without capabilities */
       log(L_WARN "%s: Capability related error received, retry with capabilities disabled", p->p.name);
-      conn->bgp->start_state = BSS_CONNECT_NOCAP;
-      delay = 0;
+      p->start_state = BSS_CONNECT_NOCAP;
+      err = 0;
     }
 #endif
 
-  if (delay) bgp_update_startup_delay(conn->bgp, conn, code, subcode);
   bgp_conn_enter_close_state(conn);
   bgp_schedule_packet(conn, PKT_SCHEDULE_CLOSE);
+
+  if (err) 
+    {
+      bgp_update_startup_delay(p);
+      bgp_stop(p, 0);
+    }
 }
 
 static void
