@@ -76,6 +76,9 @@
 #include <stdlib.h>
 #include "ospf.h"
 
+
+static void ospf_rt_notify(struct proto *p, net * n, rte * new, rte * old UNUSED, ea_list * attrs);
+static void ospf_ifa_notify(struct proto *p, unsigned flags, struct ifa *a);
 static int ospf_rte_better(struct rte *new, struct rte *old);
 static int ospf_rte_same(struct rte *new, struct rte *old);
 static void ospf_disp(timer *timer);
@@ -124,6 +127,9 @@ ospf_start(struct proto *p)
   po->disp_timer->hook = ospf_disp;
   po->disp_timer->recurrent = po->tick;
   tm_start(po->disp_timer, 1);
+  po->lsab_size = 256;
+  po->lsab_used = 0;
+  po->lsab = mb_alloc(p->pool, po->lsab_size);
   init_list(&(po->iface_list));
   init_list(&(po->area_list));
   fib_init(&po->rtf, p->pool, sizeof(ort), 16, ospf_rt_initort);
@@ -142,6 +148,7 @@ ospf_start(struct proto *p)
     oa = mb_allocz(p->pool, sizeof(struct ospf_area));
     add_tail(&po->area_list, NODE oa);
     po->areano++;
+    oa->ac = ac;
     oa->stub = ac->stub;
     oa->areaid = ac->areaid;
     oa->rt = NULL;
@@ -202,11 +209,11 @@ ospf_dump(struct proto *p)
   {
     OSPF_TRACE(D_EVENTS, "Interface: %s", (ifa->iface ? ifa->iface->name : "(null)"));
     OSPF_TRACE(D_EVENTS, "state: %u", ifa->state);
-    OSPF_TRACE(D_EVENTS, "DR:  %I", ifa->drid);
-    OSPF_TRACE(D_EVENTS, "BDR: %I", ifa->bdrid);
+    OSPF_TRACE(D_EVENTS, "DR:  %R", ifa->drid);
+    OSPF_TRACE(D_EVENTS, "BDR: %R", ifa->bdrid);
     WALK_LIST(n, ifa->neigh_list)
     {
-      OSPF_TRACE(D_EVENTS, "  neighbor %I in state %u", n->rid, n->state);
+      OSPF_TRACE(D_EVENTS, "  neighbor %R in state %u", n->rid, n->state);
     }
   }
 
@@ -224,8 +231,10 @@ ospf_init(struct proto_config *c)
   p->import_control = ospf_import_control;
   p->make_tmp_attrs = ospf_make_tmp_attrs;
   p->store_tmp_attrs = ospf_store_tmp_attrs;
+  p->accept_ra_types = RA_OPTIMAL;
   p->rt_notify = ospf_rt_notify;
   p->if_notify = ospf_iface_notify;
+  p->ifa_notify = ospf_ifa_notify;
   p->rte_better = ospf_rte_better;
   p->rte_same = ospf_rte_same;
 
@@ -300,8 +309,7 @@ schedule_rt_lsa(struct ospf_area *oa)
 {
   struct proto *p = &oa->po->proto;
 
-  OSPF_TRACE(D_EVENTS, "Scheduling RT lsa origination for area %I.",
-	     oa->areaid);
+  OSPF_TRACE(D_EVENTS, "Scheduling RT lsa origination for area %R.", oa->areaid);
   oa->origrt = 1;
 }
 
@@ -428,7 +436,7 @@ ospf_shutdown(struct proto *p)
   return PS_DOWN;
 }
 
-void
+static void
 ospf_rt_notify(struct proto *p, net * n, rte * new, rte * old UNUSED,
 	       ea_list * attrs)
 {
@@ -472,6 +480,25 @@ ospf_rt_notify(struct proto *p, net * n, rte * new, rte * old UNUSED,
   }
 }
 
+static void
+ospf_ifa_notify(struct proto *p, unsigned flags, struct ifa *a)
+{
+  struct proto_ospf *po = (struct proto_ospf *) p;
+  struct ospf_iface *ifa;
+  
+  if ((a->flags & IA_SECONDARY) || (a->flags & IA_UNNUMBERED))
+    return;
+
+  WALK_LIST(ifa, po->iface_list)
+    {
+      if (ifa->iface == a->iface)
+	{
+	  schedule_rt_lsa(ifa->oa);
+	  return;
+	}
+    }
+}
+
 static void
 ospf_get_status(struct proto *p, byte * buf)
 {
@@ -602,9 +629,31 @@ ospf_reconfigure(struct proto *p, struct proto_config *c)
     if (!oa)
       return 0;
 
+    oa->ac = newac;
     oa->stub = newac->stub;
     if (newac->stub && (oa->areaid == 0)) oa->stub = 0;
 
+    /* Check stubnet_list */
+    struct ospf_stubnet_config *oldsn = HEAD(oldac->stubnet_list);
+    struct ospf_stubnet_config *newsn = HEAD(newac->stubnet_list);
+
+    while (((NODE(oldsn))->next != NULL) && ((NODE(newsn))->next != NULL))
+      {
+	if (!ipa_equal(oldsn->px.addr, newsn->px.addr) ||
+	    (oldsn->px.len != newsn->px.len) ||
+	    (oldsn->hidden != newsn->hidden) ||
+	    (oldsn->summary != newsn->summary) ||
+	    (oldsn->cost != newsn->cost))
+	  break;
+
+	oldsn = (struct ospf_stubnet_config *)(NODE(oldsn))->next;
+	newsn = (struct ospf_stubnet_config *)(NODE(newsn))->next;
+      }
+
+    /* If there is no change, both pointers should be NULL */
+    if (((NODE(oldsn))->next) != ((NODE(newsn))->next))
+      schedule_rt_lsa(oa);
+
     /* Change net_list */
     FIB_WALK(&oa->net_fib, nf)	/* First check if some networks are deleted */
     {
@@ -903,7 +952,7 @@ ospf_sh(struct proto *p)
 
   WALK_LIST(oa, po->area_list)
   {
-    cli_msg(-1014, "\tArea: %I (%u) %s", oa->areaid, oa->areaid,
+    cli_msg(-1014, "\tArea: %R (%u) %s", oa->areaid, oa->areaid,
 	    oa->areaid == 0 ? "[BACKBONE]" : "");
     ifano = 0;
     nno = 0;
@@ -1038,7 +1087,7 @@ show_lsa_router(struct top_hash_entry *he)
 
   for (i = 0; i < rt->links; i++)
     if (rr[i].type == LSART_PTP)
-      cli_msg(-1016, "\t\trouter %I metric %u ", rr[i].id, rr[i].metric);
+      cli_msg(-1016, "\t\trouter %R metric %u ", rr[i].id, rr[i].metric);
 
   for (i = 0; i < rt->links; i++)
     if (rr[i].type == LSART_NET)
@@ -1075,10 +1124,10 @@ show_lsa_network(struct top_hash_entry *he)
 
   cli_msg(-1016, "");
   cli_msg(-1016, "\tnetwork %I/%d", ipa_and(ipa_from_u32(lsa->id), ln->netmask), ipa_mklen(ln->netmask));
-  cli_msg(-1016, "\t\tdr %I", lsa->rt);
+  cli_msg(-1016, "\t\tdr %R", lsa->rt);
 
   for (i = 0; i < max; i++)
-    cli_msg(-1016, "\t\trouter %I", rts[i]);
+    cli_msg(-1016, "\t\trouter %R", rts[i]);
 }
 
 static inline void
@@ -1093,7 +1142,7 @@ show_lsa_sum_net(struct top_hash_entry *he)
 static inline void
 show_lsa_sum_rt(struct top_hash_entry *he)
 {
-  cli_msg(-1016, "\t\txrouter %I", he->lsa.id);
+  cli_msg(-1016, "\t\txrouter %R", he->lsa.id);
 }
 
 
@@ -1157,7 +1206,7 @@ ospf_sh_state(struct proto *p, int verbose)
     if (last_area != hea[i]->oa->areaid)
     {
       cli_msg(-1016, "");
-      cli_msg(-1016, "area %I", hea[i]->oa->areaid);
+      cli_msg(-1016, "area %R", hea[i]->oa->areaid);
       last_area = hea[i]->oa->areaid;
       last_rt = 0xFFFFFFFF;
     }
@@ -1165,7 +1214,7 @@ ospf_sh_state(struct proto *p, int verbose)
     if ((hea[i]->lsa.rt != last_rt) && (hea[i]->lsa.type != LSA_T_NET))
     {
       cli_msg(-1016, "");
-      cli_msg(-1016, (hea[i]->lsa.type != LSA_T_EXT) ? "\trouter %I" : "\txrouter %I", hea[i]->lsa.rt);
+      cli_msg(-1016, (hea[i]->lsa.type != LSA_T_EXT) ? "\trouter %R" : "\txrouter %R", hea[i]->lsa.rt);
       last_rt = hea[i]->lsa.rt;
     }
 
