@@ -99,6 +99,14 @@ struct area_net
   u32 metric;
 };
 
+struct ospf_stubnet_config
+{
+  node n;
+  struct prefix px;
+  int hidden, summary;
+  u32 cost;
+};
+
 struct ospf_area_config
 {
   node n;
@@ -107,11 +115,12 @@ struct ospf_area_config
   list patt_list;
   list vlink_list;
   list net_list;
+  list stubnet_list;
 };
 
 struct obits
 {
-#ifdef _BIG_ENDIAN
+#ifdef CPU_BIG_ENDIAN
   u8 unused2:2;
   u8 dc:1;
   u8 ea:1;
@@ -255,7 +264,7 @@ struct ospf_hello_packet
 
 struct immsb
 {
-#ifdef _BIG_ENDIAN
+#ifdef CPU_BIG_ENDIAN
   u8 padding:5;
   u8 i:1;
   u8 m:1;
@@ -311,7 +320,7 @@ struct ospf_lsa_header
 
 struct vebb
 {
-#ifdef _BIG_ENDIAN
+#ifdef CPU_BIG_ENDIAN
   u8 padding:5;
   u8 v:1;
   u8 e:1;
@@ -375,7 +384,7 @@ struct ospf_lsa_ext
 
 struct ospf_lsa_ext_etos 
 {
-#ifdef _BIG_ENDIAN
+#ifdef CPU_BIG_ENDIAN
   u8 ebit:1;
   u8 tos:7;
   u8 padding1;
@@ -391,7 +400,7 @@ struct ospf_lsa_ext_etos
 #define METRIC_MASK 0x00FFFFFF
 struct ospf_lsa_sum_tos 
 {
-#ifdef _BIG_ENDIAN
+#ifdef CPU_BIG_ENDIAN
   u8 tos;
   u8 padding1;
   u16 padding2;
@@ -523,6 +532,7 @@ struct ospf_area
 {
   node n;
   u32 areaid;
+  struct ospf_area_config *ac;	/* Related area config */
   int origrt;			/* Rt lsa origination scheduled? */
   struct top_hash_entry *rt;	/* My own router LSA */
   list cand;			/* List of candidates for RT calc. */
@@ -550,6 +560,8 @@ struct proto_ospf
   int rfc1583;			/* RFC1583 compatibility */
   int ebit;			/* Did I originate any ext lsa? */
   struct ospf_area *backbone;	/* If exists */
+  void *lsab;			/* LSA buffer used when originating router LSAs */
+  int lsab_size, lsab_used;
 };
 
 struct ospf_iface_patt
@@ -585,14 +597,14 @@ int ospf_import_control(struct proto *p, rte **new, ea_list **attrs,
 			struct linpool *pool);
 struct ea_list *ospf_make_tmp_attrs(struct rte *rt, struct linpool *pool);
 void ospf_store_tmp_attrs(struct rte *rt, struct ea_list *attrs);
-void ospf_rt_notify(struct proto *p, net *n, rte *new, rte *old,
-		    ea_list * attrs);
 void schedule_rt_lsa(struct ospf_area *oa);
 void schedule_rtcalc(struct proto_ospf *po);
 void schedule_net_lsa(struct ospf_iface *ifa);
 void ospf_sh_neigh(struct proto *p, char *iff);
 void ospf_sh(struct proto *p);
 void ospf_sh_iface(struct proto *p, char *iff);
+void ospf_sh_state(struct proto *p, int verbose);
+
 
 #define EA_OSPF_METRIC1	EA_CODE(EAP_OSPF, 0)
 #define EA_OSPF_METRIC2	EA_CODE(EAP_OSPF, 1)
