@@ -9,14 +9,19 @@
 #ifndef _BIRD_PIPE_H_
 #define _BIRD_PIPE_H_
 
+#define PIPE_OPAQUE 0
+#define PIPE_TRANSPARENT 1
+
 struct pipe_config {
   struct proto_config c;
   struct rtable_config *peer;		/* Table we're connected to */
+  int mode;				/* PIPE_OPAQUE or PIPE_TRANSPARENT */
 };
 
 struct pipe_proto {
   struct proto p;
   struct rtable *peer;
+  int mode;				/* PIPE_OPAQUE or PIPE_TRANSPARENT */
   struct pipe_proto *phantom;
 };
 
