@@ -268,7 +268,7 @@ rip_rte_update_if_better(rtable *tab, net *net, struct proto *p, rte *new)
   if (!old || p->rte_better(new, old) ||
       (ipa_equal(old->attrs->from, new->attrs->from) &&
       (old->u.rip.metric != new->u.rip.metric)) )
-    rte_update(tab, net, p, new);
+    rte_update(tab, net, p, p, new);
 }
 
 /*
@@ -946,6 +946,7 @@ rip_rte_remove(net *net UNUSED, rte *rte)
 void
 rip_init_instance(struct proto *p)
 {
+  p->accept_ra_types = RA_OPTIMAL;
   p->if_notify = rip_if_notify;
   p->rt_notify = rip_rt_notify;
   p->import_control = rip_import_control;
