@@ -500,7 +500,7 @@ nl_send_route(struct krt_proto *p, rte *e, int new)
   bzero(&r.r, sizeof(r.r));
   r.h.nlmsg_type = new ? RTM_NEWROUTE : RTM_DELROUTE;
   r.h.nlmsg_len = NLMSG_LENGTH(sizeof(struct rtmsg));
-  r.h.nlmsg_flags = NLM_F_REQUEST | NLM_F_ACK | (new ? NLM_F_CREATE|NLM_F_REPLACE : 0);
+  r.h.nlmsg_flags = NLM_F_REQUEST | NLM_F_ACK | (new ? NLM_F_CREATE|NLM_F_EXCL : 0);
 
   r.r.rtm_family = BIRD_AF;
   r.r.rtm_dst_len = net->n.pxlen;
@@ -540,22 +540,11 @@ nl_send_route(struct krt_proto *p, rte *e, int new)
 void
 krt_set_notify(struct krt_proto *p, net *n UNUSED, rte *new, rte *old)
 {
-  if (old && new)
-    {
-      /*
-       *  We should check whether priority and TOS is identical as well,
-       *  but we don't use these and default value is always equal to default value. :-)
-       */
-      nl_send_route(p, new, 1);
-    }
-  else
-    {
-      if (old)
-	nl_send_route(p, old, 0);
+  if (old)
+    nl_send_route(p, old, 0);
 
-      if (new)
-	nl_send_route(p, new, 1);
-    }
+  if (new)
+    nl_send_route(p, new, 1);
 }
 
 static struct iface *
@@ -693,12 +682,30 @@ nl_parse_route(struct nlmsghdr *h, int scan)
 	  ra.dest = RTD_ROUTER;
 	  memcpy(&ra.gw, RTA_DATA(a[RTA_GATEWAY]), sizeof(ra.gw));
 	  ipa_ntoh(ra.gw);
-	  ng = neigh_find(&p->p, &ra.gw, 0);
-	  if (ng && ng->scope)
-	    ra.iface = ng->iface;
+
+	  if (i->rtm_flags & RTNH_F_ONLINK)
+	    {
+	      /* route with 'onlink' attribute */
+	      ra.iface = if_find_by_index(oif);
+	      if (ra.iface == NULL)
+		{
+		  log(L_WARN "Kernel told us to use unknown interface %u for %I/%d",
+		      oif, net->n.prefix, net->n.pxlen);
+		  return;
+		}
+	    }
 	  else
-	    /* FIXME: Remove this warning? Handle it somehow... */
-	    log(L_WARN "Kernel told us to use non-neighbor %I for %I/%d", ra.gw, net->n.prefix, net->n.pxlen);
+	    {
+	      /* standard route */
+	      ng = neigh_find(&p->p, &ra.gw, 0);
+	      if (ng && ng->scope)
+		ra.iface = ng->iface;
+	      else
+		{
+		  log(L_WARN "Kernel told us to use non-neighbor %I for %I/%d", ra.gw, net->n.prefix, net->n.pxlen);
+		  return;
+		}
+	    }
 	}
       else
 	{
