@@ -30,6 +30,17 @@
 #include "lib/unix.h"
 #include "lib/sysio.h"
 
+/* Maximum number of calls of tx handler for one socket in one
+ * select iteration. Should be small enough to not monopolize CPU by
+ * one protocol instance.
+ */
+#define MAX_STEPS 4
+
+/* Maximum number of calls of rx handler for all sockets in one select
+   iteration. RX callbacks are often much more costly so we limit
+   this to gen small latencies */
+#define MAX_RX_STEPS 4
+
 /*
  *	Tracked Files
  */
@@ -199,10 +210,6 @@ timer *
 tm_new(pool *p)
 {
   timer *t = ralloc(p, &tm_class);
-  t->hook = NULL;
-  t->data = NULL;
-  t->randomize = 0;
-  t->expires = 0;
   return t;
 }
 
@@ -491,6 +498,7 @@ tm_format_reltime(char *x, bird_clock_t t)
 
 static list sock_list;
 static struct birdsock *current_sock;
+static struct birdsock *stored_sock;
 static int sock_recalc_fdsets_p;
 
 static inline sock *
@@ -539,6 +547,8 @@ sk_free(resource *r)
       close(s->fd);
       if (s == current_sock)
 	current_sock = sk_next(s);
+      if (s == stored_sock)
+	stored_sock = sk_next(s);
       rem_node(&s->n);
       sock_recalc_fdsets_p = 1;
     }
@@ -589,21 +599,9 @@ sk_new(pool *p)
 {
   sock *s = ralloc(p, &sk_class);
   s->pool = p;
-  s->data = NULL;
-  s->saddr = s->daddr = IPA_NONE;
-  s->sport = s->dport = 0;
+  // s->saddr = s->daddr = IPA_NONE;
   s->tos = s->ttl = -1;
-  s->iface = NULL;
-  s->rbuf = NULL;
-  s->rx_hook = NULL;
-  s->rbsize = 0;
-  s->tbuf = NULL;
-  s->tx_hook = NULL;
-  s->tbsize = 0;
-  s->err_hook = NULL;
   s->fd = -1;
-  s->rbuf_alloc = s->tbuf_alloc = NULL;
-  s->password = NULL;
   return s;
 }
 
@@ -703,7 +701,13 @@ sk_setup(sock *s)
   if ((s->tos >= 0) && setsockopt(fd, SOL_IP, IP_TOS, &s->tos, sizeof(s->tos)) < 0)
     WARN("IP_TOS");
 #endif
-  
+
+#ifdef IPV6
+  int v = 1;
+  if ((s->flags & SKF_V6ONLY) && setsockopt(fd, IPPROTO_IPV6, IPV6_V6ONLY, &v, sizeof(v)) < 0)
+    WARN("IPV6_V6ONLY");
+#endif
+
   if (s->ttl >= 0)
     err = sk_set_ttl_int(s);
   else
@@ -921,7 +925,8 @@ sk_open(sock *s)
 	}
       fill_in_sockaddr(&sa, s->saddr, port);
 #ifdef CONFIG_SKIP_MC_BIND
-      if (type == SK_IP && bind(fd, (struct sockaddr *) &sa, sizeof(sa)) < 0)
+      if ((type != SK_UDP_MC) && (type != SK_IP_MC) &&
+	  bind(fd, (struct sockaddr *) &sa, sizeof(sa)) < 0)
 #else
       if (bind(fd, (struct sockaddr *) &sa, sizeof(sa)) < 0)
 #endif
@@ -1074,6 +1079,29 @@ sk_maybe_write(sock *s)
     }
 }
 
+int
+sk_rx_ready(sock *s)
+{
+  fd_set rd, wr;
+  struct timeval timo;
+  int rv;
+
+  FD_ZERO(&rd);
+  FD_ZERO(&wr);
+  FD_SET(s->fd, &rd);
+
+  timo.tv_sec = 0;
+  timo.tv_usec = 0;
+
+ redo:
+  rv = select(s->fd+1, &rd, &wr, NULL, &timo);
+  
+  if ((rv < 0) && (errno == EINTR || errno == EAGAIN))
+    goto redo;
+
+  return rv;
+}
+
 /**
  * sk_send - send data to a socket
  * @s: socket
@@ -1242,6 +1270,9 @@ io_init(void)
   srandom((int) now_real);
 }
 
+static int short_loops = 0;
+#define SHORT_LOOP_MAX 10
+
 void
 io_loop(void)
 {
@@ -1320,8 +1351,8 @@ io_loop(void)
 	}
 
       /* And finally enter select() to find active sockets */
-
       hi = select(hi+1, &rd, &wr, NULL, &timo);
+
       if (hi < 0)
 	{
 	  if (errno == EINTR || errno == EAGAIN)
@@ -1330,30 +1361,68 @@ io_loop(void)
 	}
       if (hi)
 	{
-	  current_sock = SKIP_BACK(sock, n, HEAD(sock_list));	/* guaranteed to be non-empty */
+	  /* guaranteed to be non-empty */
+	  current_sock = SKIP_BACK(sock, n, HEAD(sock_list));
+
 	  while (current_sock)
 	    {
 	      sock *s = current_sock;
 	      int e;
-	      if (FD_ISSET(s->fd, &rd) && s->rx_hook)
+	      int steps;
+
+	      steps = MAX_STEPS;
+	      if ((s->type >= SK_MAGIC) && FD_ISSET(s->fd, &rd) && s->rx_hook)
 		do
 		  {
+		    steps--;
 		    e = sk_read(s);
 		    if (s != current_sock)
 		      goto next;
 		  }
-		while (e && s->rx_hook);
+		while (e && s->rx_hook && steps);
+
+	      steps = MAX_STEPS;
 	      if (FD_ISSET(s->fd, &wr))
 		do
 		  {
+		    steps--;
 		    e = sk_write(s);
 		    if (s != current_sock)
 		      goto next;
 		  }
-		while (e);
+		while (e && steps);
 	      current_sock = sk_next(s);
 	    next: ;
 	    }
+
+	  short_loops++;
+	  if (events && (short_loops < SHORT_LOOP_MAX))
+	    continue;
+	  short_loops = 0;
+
+	  int count = 0;
+	  current_sock = stored_sock;
+	  if (current_sock == NULL)
+	    current_sock = SKIP_BACK(sock, n, HEAD(sock_list));
+
+	  while (current_sock && count < MAX_RX_STEPS)
+	    {
+	      sock *s = current_sock;
+	      int e;
+	      int steps;
+
+	      if ((s->type < SK_MAGIC) && FD_ISSET(s->fd, &rd) && s->rx_hook)
+		{
+		  count++;
+		  e = sk_read(s);
+		  if (s != current_sock)
+		      goto next2;
+		}
+	      current_sock = sk_next(s);
+	    next2: ;
+	    }
+
+	  stored_sock = current_sock;
 	}
     }
 }
