@@ -115,7 +115,7 @@ read_config(void)
       else
 	die("Unable to open configuration file %s: %m", config_name);
     }
-  config_commit(conf);
+  config_commit(conf, RECONFIG_HARD);
 }
 
 void
@@ -133,11 +133,11 @@ async_config(void)
       config_free(conf);
     }
   else
-    config_commit(conf);
+    config_commit(conf, RECONFIG_HARD);
 }
 
 void
-cmd_reconfig(char *name)
+cmd_reconfig(char *name, int type)
 {
   struct config *conf;
 
@@ -154,7 +154,7 @@ cmd_reconfig(char *name)
     }
   else
     {
-      switch (config_commit(conf))
+      switch (config_commit(conf, type))
 	{
 	case CONF_DONE:
 	  cli_msg(3, "Reconfigured.");
@@ -178,22 +178,44 @@ cmd_reconfig(char *name)
 static sock *cli_sk;
 static char *path_control_socket = PATH_CONTROL_SOCKET;
 
-int
+
+static void
 cli_write(cli *c)
 {
   sock *s = c->priv;
 
-  if (c->tx_pos)
+  while (c->tx_pos)
     {
       struct cli_out *o = c->tx_pos;
+
+      int len = o->wpos - o->outpos;
       s->tbuf = o->outpos;
-      if (sk_send(s, o->wpos - o->outpos) > 0)
-	{
-	  c->tx_pos = o->next;
-	  ev_schedule(c->event);
-	}
+      o->outpos = o->wpos;
+
+      if (sk_send(s, len) <= 0)
+	return;
+
+      c->tx_pos = o->next;
     }
-  return !c->tx_pos;
+
+  /* Everything is written */
+  s->tbuf = NULL;
+  cli_written(c);
+}
+
+void
+cli_write_trigger(cli *c)
+{
+  sock *s = c->priv;
+
+  if (s->tbuf == NULL)
+    cli_write(c);
+}
+
+static void
+cli_tx(sock *s)
+{
+  cli_write(s->data);
 }
 
 int
@@ -232,15 +254,6 @@ cli_rx(sock *s, int size UNUSED)
   return 0;
 }
 
-static void
-cli_tx(sock *s)
-{
-  cli *c = s->data;
-
-  if (cli_write(c))
-    cli_written(c);
-}
-
 static void
 cli_err(sock *s, int err)
 {
