@@ -16,7 +16,7 @@ struct pool;
 void async_config(void);
 void async_dump(void);
 void async_shutdown(void);
-void cmd_reconfig(char *name);
+void cmd_reconfig(char *name, int type);
 
 /* io.c */
 
